<?php
/**                               ______________________________________________
*                          o O   |                                              |
*                 (((((  o      <  JDom Class - Cook librarie    (100% Vitamin) |
*                ( o o )         |______________________________________________|
* --------oOOO-----(_)-----OOOo---------------------------------- www.j-cook.pro --- +
* @version		1.0.0
* @package		Cook
* @subpackage	JDom
* @license		GNU General Public License
* @author		100% Vitamin - Jocelyn HUARD
*
*	-> You can reuse this framework for all purposes. Keep this signature. <-
*
* /!\  Joomla! is free software.
* This version may have been modified pursuant to the GNU General Public License,
* and as distributed it includes or is derivative of works licensed under the
* GNU General Public License or other free or open source software licenses.
*
*             .oooO  Oooo.     See COPYRIGHT.php for copyright notices and details.
*             (   )  (   )
* -------------\ (----) /----------------------------------------------------------- +
*               \_)  (_/
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );


define("BR", "<br />");
define("LN", "\n");





/*
 * JDom Framework is an abstraction between your component and the HTML renderer (of your choice)
 *
 * 	Rewrite inside the element classes files you want to change, or override them (see below)
 * 	Using JDom in your component, you'll be able to upgrade all your component DOM possibilities in seconds...
 *
 *  See documentation at www.jcook.pro
 *
 *
 *	OVERRIDES :
 * 	You can place the files you want to override wherever you prefers see the $searches array;
 *
 *	in the app site client	ie : components/com_mycomponent/dom/html/form/input/select.php
 * 	in the template			ie : templates/my_template/html/com_mycomponent/dom/html/form/input/select.php
 *  in the template view	ie : templates/my_template/html/com_mycomponent/my_view/dom/html/form/input/select.php
 *	and more ...
 *
 *	The search array defines the order of priority for overriding
 *
 *  JDom is 100% compatible for all Joomla! versions since 1.5
 *
 */
class JDom extends JObject
{
	var $path;
	var $options;
	var $app;

	/*
	 * Constuctor
	 *
	 * 	@namespace 	: requested class
	 *  @options	: Configuration
	 *
	 */
	function __construct($args)
	{
		$this->arg('namespace'	, 0, $args);
		$this->arg('options'	, 1, $args);

		$this->app = JFactory::getApplication();
	}

	/*
	 * Static function to render a DOM object/input
	 *
	 */
	static function _($namespace, $options = array())
	{
		$args = func_get_args();
		$class = self::getClass($namespace, $args);

		return $class->build($args);
	}

	function getClass($namespace, $args)
	{

		$domType = self::getName($namespace);
		$class = self::getInstance($args, $domType);

		return $class;
	}

	function build($args)
	{

		$class = $this->getInstance($args);

		//ACL Access
		if (!$class->access())
			return "";

		//Assets implementations
		$class->addScript();

		$lastCall = ($this->level == count(explode('.', $this->namespace)));

		if ($lastCall)
			$this->buildJS();		//Scripts inline


		$html = $class->build($args);  //Recursivity

		if ($lastCall)
			$html = $class->embedLink($html);



		$html = $class->parse($html);   //Recursivity and parsing





		$indent = $this->getOption('indent');
		if ($indent && ($this->level == 1))
		{
			$html = $this->indent($html, $indent);
		}

		return $html;

	}

	/*
	 * Abstract
	 */
	function buildJS()	{}

	function getInstance($args, $name = null)
	{

		if ($name)	//Static call  - Level = 0
		{
			$relativeName = $name . ".php";
			$className = "JDom" . ucfirst($name);
			$path = $name;
			$error = self::includeFile($relativeName, $className, true);

		}
		else
		{
			$name = $this->getName();
			$path = $this->path .DS. $name;


			$relativeName = $path . ".php";
			$className = get_class($this) . ucfirst($name);


			$error = $this->includeFile($relativeName, $className);

		}

		if ($error)
			return $error;

		$class = new $className($args);
		$class->path = $path;

		return $class;
	}


	/*
	 * Search the appropriate class file, depending on context
	 *
	 */
	function includeFile($relativeName, $className, $static = false)
	{
		if ($static)
			$file = self::searchFile($relativeName, $static);
		else
			$file = $this->searchFile($relativeName, $static);


		if (!$file)
		{
			//Not founded
			JError::setErrorHandling(E_ERROR, 'die'); //force error type to die
			$error = JError::raiseError( 500, JText::_('Unable to load JDom class : ') . $className);

			return $error;

		}

		require_once($file);
	}


	function searchFile($relativeName, $static = false)
	{
		// Defines the priority ORDER for classes FALLBACKS
		// TODO : Comment some lines, or change order depending on how you want to use this functionnality


		if (!$static)
			$searches = $this->getOption('searches');


		if (!isset($searches))
		{
			$searches = array(
					'template.view',		// 	Files on the view directory of the template -> Filter on particular view
					'template.component',	// 	Files on the component directory of the template -> Filter for this component
					'template',				// 	Files on the root directory of the template
					'client.view',			//	Files on the component view directory -> Search in the current client side (front or back)
					'client',				//	Files on the component root directory -> Search in the current client side (front or back)
					'front.view',			//	Files on the FRONT component view directory (Site client)
					'front',				//	Files on the FRONT component root directory (Site client)
					'back.view',			//	Files on the BACK component view directory (Administrator client)
					'back',					//	Files on the BACK component root directory (Administrator client)

					);
		}



		//Extension
		if (!$static)
			$extension = $this->getExtension();
		else
			$extension = JRequest::getVar('option');

		//View
		if (!$static)
			$view = $this->getView();
		else
			$view = JRequest::getVar('view');


		$app = JFactory::getApplication();
		$tmpl = $app->getTemplate();
		$tmplPath = JPATH_SITE .DS. 'templates' .DS. $tmpl .DS. 'html';

		if ($searches)
		foreach($searches as $search)
		{
			switch($search)
			{
				case 'template.view';
					$path = $tmplPath .DS. $extension .DS. $view;
					break;

				case 'template.component';
					$path = $tmplPath .DS. $extension;
					break;

				case 'template';
					$path = $tmplPath;
					break;

				case 'client.view';
					$path = JPATH_COMPONENT .DS. 'views' .DS. $view;
					break;

				case 'client';
					$path = JPATH_COMPONENT;
					break;

				case 'front.view';
					$path = JPATH_SITE .DS. 'components' .DS. $extension .DS. 'views' .DS. $view;
					break;

				case 'front';
					$path = JPATH_SITE .DS. 'components' .DS. $extension;
					break;

				case 'back.view';
					$path = JPATH_ADMINISTRATOR .DS. 'components' .DS. $extension .DS. 'views' .DS. $view;
					break;

				case 'back';
					$path = JPATH_ADMINISTRATOR .DS. 'components' .DS. $extension;
					break;


				default:
					$path = $search;		//Custom path
					break;

			}

			$completePath = $path .DS. 'dom' .DS. $relativeName;

			if (file_exists($completePath))
			{
				return $completePath;
			}
		}




		return null;


	}

	function arg($name, $i = null, $args = array(), $fallback = null)
	{
		$optionValue = $this->getOption($name);

		if ($optionValue !== null)
			$this->$name = $this->options[$name];
		else if (($i !== null) && (count($args) > $i))
			if ($args[$i] !== null)
				$this->$name = $args[$i];
			else
				$this->$name = $fallback;

		if (!isset($this->$name) && ($fallback !== null))
			$this->$name = $fallback;


		if ($optionValue)
			$this->options[$name] = $this->$name;

	}

	function isArg($varname)
	{
		if (isset($this->$varname) || (is_array($this->options) && (in_array($varname, array_keys($this->options)))))
			return true;
		else
			return false;
	}

	function getOption($name)
	{
		if (($name != 'options') && (is_array($this->options)) && (in_array($name, array_keys($this->options))))
			return $this->options[$name];

		return null;
	}

	/*
	 * Abstract
	 */
	function jsonArgs($args = array())
	{
		return json_encode($args);

	}

	function getName($namespace = null)
	{
		if ($namespace)		//Static call  - Level = 0
		{
			$parts = explode(".", $namespace);

			$name = $parts[0];
			if (!$name)
				$name = 'html'; 	//Fallback


		}
		else
		{
			$parts = explode(".", $this->namespace);
			if (count($parts) > $this->level)
				$name = $parts[$this->level];
			else
				$name = $this->fallback;

		}

		return $name;


	}

	function indent($contents, $indent)
	{
		if (is_int($indent))
		{
			$indentStr = "";
			for($i = 0 ; $i < $indent ; $i++)
				$indentStr .= "	";
		}
		else
			$indentStr = $indent;



		$lines = explode("\n", $contents);
		$indentedLines = array();

		foreach($lines as $line)
		{
			if (trim($line) != "") //Don't indent line if empty
				$line = $indentStr . $line;

			$indentedLines[] =  $line;
		}

		return implode("\n", $indentedLines);
	}

	function getExtension()
	{
		$extension = $this->getOption('extension');

		if (!$extension)
			$extension = JRequest::getVar('option');

		return $extension;
	}

	function getView()
	{
		$view = $this->getOption('view');

		if (!$view)
			$view = JRequest::getVar('view');

		return $view;
	}




	/*
	 * 	Abstract
	 */
	function parseVars()
	{
		return array();
	}

	function parse($pattern)
	{
		$vars = $this->parseVars();

		$html = $pattern;

		if (isset($vars) && count($vars))
		foreach($vars as $key => $value)
		{
			$html = preg_replace("/<%" . strtoupper($key) . "%>/", $value, $html);
		}

		return $html;

	}

	/*
	 * Parse a string with JText
	 * Accepts a composed string ie : "[MY_FIRST_STRING], [MY_SECOND_STRING] : "
	 */
	function JText($text)
	{

		if (preg_match("/\[([A-Z0-9_]+)\]/", $text))
		{
			preg_match_all("/\[([A-Z0-9_]+)\]/", $text, $results);
			foreach($results[1] as $string)
			{
				$translated = JText::_($string);
				$text = preg_replace("/\[(" . $string . ")\]/", JText::_($string), $text);
			}
		}
		else
			$text = JText::_($text);

		return $text;

	}

	function addScript($assetPath = null)
	{

		if ((!$assetPath) && (!isset($this->assetName)))
			return;


		if ($assetPath)
		{
			$relativeName = 'assets' .DS. $assetPath;
		}
		else
		{
			$name = $this->assetName;
			$relativeName = 'assets' .DS. $name . DS. 'js' .DS . $name . '.js';
		}

		$jsFile = $this->searchFile($relativeName, false);
		if ($jsFile)
		{
			$doc =& JFactory::getDocument();

			//Works with Mootools
			JHTML::_('behavior.mootools');

			//Add JDom Js main class (before)
			$jDomClass = 'assets' .DS. 'jdom.js';
			$jsFileMain = $this->searchFile($jDomClass, false);
			if ($jsFileMain)
			{
				$doc->addScript(self::pathToUrl($jsFileMain));
			}

			//Loads the asset script
			$doc->addScript(self::pathToUrl($jsFile));

		}

	}

	function addScriptInline($script)
	{
		$doc =& JFactory::getDocument();
		$doc->addScriptDeclaration($script);
	}

	function pathToUrl($path, $raw = false)
	{
		$JPATH_SITE = JPATH_SITE;
		$path = str_replace("\\", "/", $path);
		$JPATH_SITE = str_replace("\\", "/", $JPATH_SITE);

		$escaped = preg_replace("/\//", "\/", $JPATH_SITE);
		$relUrl = preg_replace("/^" . $escaped . "/", "", $path);

		if ($raw)
			return $relUrl;


		return JURI::root(true) . $relUrl;
	}


	function jVersion($ver, $comp = '>=')
	{
		return version_compare(JVERSION, $ver, $comp);
	}

	function adminTemplate()
	{
		 //TODO

		if ($this->jVersion('1.6'))
			return 'bluestork';
		else
			return 'khepri';
	}

	function systemImagesDir()
	{
		if ($this->jVersion('1.6'))
			$dir = 'templates' .DS. $this->adminTemplate() .DS. 'images' .DS. 'admin';
		else
			$dir = "images";

		if ($this->app->isSite())
			$dir = administrator .DS . $dir;

		return $dir;
	}


	function extensionDir()
	{
		return JPATH_ADMINISTRATOR .DS. 'components' .DS. $this->getExtension();
	}

	function domUrl()
	{
		$url = self::pathToUrl($this->extensionDir() . '/dom');
		return $url;
	}

	function assetImage($imageName, $assetName = null)
	{
		if (!$assetName)
			$assetName = $this->getName();

		$urlImage = self::domUrl().'/assets/'. $assetName . '/images/' . $imageName;

		return $urlImage;
	}

	function htmlAssetSpriteImage($urlImage, $d)
	{
		$image = "<div style='background-image: url(" . $urlImage . ");"
			.	"width:" . $d->w . "px;"
			.	"height:" . $d->h . "px;"
			.	"background-position:-" . $d->x . "px -" . $d->y . "px;'>"
			.	"</div>";

		return $image;
	}

	function accessTask($task)
	{
		$aclAccess = $this->getOption('aclAccess');

		if ($aclAccess)
			return $this->access();

		switch ($task)
		{
			case 'new':
				$access = 'core.create';
				break;

			case 'edit':
			case 'save':
			case 'apply':
				$access = 'core.edit';
				break;

			case 'publish':
			case 'unpublish':
			case 'trash':
			case 'default_it':
				$access = 'core.edit.state';
				break;

			case 'delete':
			case 'empty_trash':
				$access = 'core.delete';
				break;

			case 'config':
				$access = 'core.manage';
				break;

			default:
				return true;
				break;

		}

		return $this->access($access);
	}

	function access($aclAccess = null)
	{
		if (!$aclAccess)
			$aclAccess = $this->getOption('aclAccess');

		if (!$aclAccess)
			return true;

		$aclAsset = $this->getOption('aclAsset');
		if (!$aclAsset)
			$aclAsset = $this->getExtension();



		$user 	=& JFactory::getUser();

		if (version_compare(JVERSION, '1.6', '<'))
		{
			$auth = $user->authorize($aclAsset, $aclAccess);
		}
		else
		{
			$auth = $user->authorise($aclAccess, $aclAsset);
		}




		return $auth;
	}
}


